@@ -2,17 +2,17 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2016 Derick Rethans                               |
+   | Copyright (c) 2002-2018 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 1.0 of the Xdebug license,    |
+   | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | http://xdebug.derickrethans.nl/license.php                           |
+   | https://xdebug.org/license.php                                       |
    | If you did not receive a copy of the Xdebug license and are unable   |
    | to obtain it through the world-wide-web, please send a note to       |
-   | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
+   | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors:  Derick Rethans <derick@xdebug.org>                         |
+   | Authors: Derick Rethans <derick@xdebug.org>                          |
    +----------------------------------------------------------------------+
  */
 
@@ -20,10 +20,10 @@
 #define PHP_XDEBUG_H
 
 #define XDEBUG_NAME       "Xdebug"
-#define XDEBUG_VERSION    "2.5.4"
+#define XDEBUG_VERSION    "2.6.0"
 #define XDEBUG_AUTHOR     "Derick Rethans"
-#define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2017 by Derick Rethans"
-#define XDEBUG_COPYRIGHT_SHORT "Copyright (c) 2002-2017"
+#define XDEBUG_COPYRIGHT  "Copyright (c) 2002-2018 by Derick Rethans"
+#define XDEBUG_COPYRIGHT_SHORT "Copyright (c) 2002-2018"
 #define XDEBUG_URL        "http://xdebug.org"
 #define XDEBUG_URL_FAQ    "http://xdebug.org/docs/faq#api"
 
@@ -94,6 +94,7 @@ PHP_FUNCTION(xdebug_disable);
 PHP_FUNCTION(xdebug_is_enabled);
 
 /* breaking functions */
+PHP_FUNCTION(xdebug_is_debugger_active);
 PHP_FUNCTION(xdebug_break);
 
 /* tracing functions */
@@ -116,6 +117,13 @@ PHP_FUNCTION(xdebug_get_profiler_filename);
 PHP_FUNCTION(xdebug_dump_aggr_profiling_data);
 PHP_FUNCTION(xdebug_clear_aggr_profiling_data);
 
+/* gc stats functions */
+PHP_FUNCTION(xdebug_start_gcstats);
+PHP_FUNCTION(xdebug_stop_gcstats);
+PHP_FUNCTION(xdebug_get_gcstats_filename);
+PHP_FUNCTION(xdebug_get_gc_run_count);
+PHP_FUNCTION(xdebug_get_gc_total_collected_roots);
+
 /* misc functions */
 PHP_FUNCTION(xdebug_dump_superglobals);
 PHP_FUNCTION(xdebug_get_headers);
@@ -123,17 +131,20 @@ PHP_FUNCTION(xdebug_memory_usage);
 PHP_FUNCTION(xdebug_peak_memory_usage);
 PHP_FUNCTION(xdebug_time_index);
 
+/* filter functions */
+PHP_FUNCTION(xdebug_set_filter);
+
 ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	int           status;
 	int           reason;
 
 	unsigned long level;
 	xdebug_llist *stack;
-	iniLONG       max_nesting_level;
-	iniLONG       max_stack_frames;
+	zend_long     max_nesting_level;
+	zend_long     max_stack_frames;
 	zend_bool     default_enable;
 	zend_bool     collect_includes;
-	iniLONG       collect_params;
+	zend_long     collect_params;
 	zend_bool     collect_return;
 	zend_bool     collect_vars;
 	zend_bool     collect_assignments;
@@ -149,11 +160,12 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	function_stack_entry *active_fse;
 	unsigned int  prev_memory;
 	char         *file_link_format;
+	char         *filename_format;
 	zend_bool     force_display_errors;
-	iniLONG       force_error_reporting;
-	iniLONG       halt_level;
+	zend_long     force_error_reporting;
+	zend_long     halt_level;
 
-	zend_bool     overload_var_dump;
+	zend_long     overload_var_dump;
 	void        (*orig_var_dump_func)(INTERNAL_FUNCTION_PARAMETERS);
 	void        (*orig_set_time_limit_func)(INTERNAL_FUNCTION_PARAMETERS);
 	void        (*orig_pcntl_exec_func)(INTERNAL_FUNCTION_PARAMETERS);
@@ -167,16 +179,16 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	char         *trace_enable_trigger_value;
 	char         *trace_output_dir;
 	char         *trace_output_name;
-	iniLONG       trace_options;
-	iniLONG       trace_format;
+	zend_long     trace_options;
+	zend_long     trace_format;
 	char         *last_exception_trace;
 	char         *last_eval_statement;
 	zend_bool     in_debug_info;
 
 	/* variable dumping limitation settings */
-	iniLONG       display_max_children;
-	iniLONG       display_max_data;
-	iniLONG       display_max_depth;
+	zend_long     display_max_children;
+	zend_long     display_max_data;
+	zend_long     display_max_depth;
 
 	zend_bool     cli_color;
 	int           output_is_tty;
@@ -191,6 +203,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	unsigned int  function_count;
 	int           dead_code_analysis_tracker_offset;
 	long          dead_code_last_start_id;
+	long          code_coverage_filter_offset;
 	char                 *previous_filename;
 	xdebug_coverage_file *previous_file;
 	char                 *previous_mark_filename;
@@ -230,7 +243,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 
 	/* remote settings */
 	zend_bool     remote_enable;  /* 0 */
-	iniLONG       remote_port;    /* 9000 */
+	zend_long     remote_port;    /* 9000 */
 	char         *remote_host;    /* localhost */
 	long          remote_mode;    /* XDEBUG_NONE, XDEBUG_JIT, XDEBUG_REQ */
 	char         *remote_handler; /* php3, gdb, dbgp */
@@ -238,8 +251,9 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	zend_bool     remote_connect_back;   /* connect back to the HTTP requestor */
 	char         *remote_log;       /* Filename to log protocol communication to */
 	FILE         *remote_log_file;  /* File handler for protocol log */
-	iniLONG       remote_cookie_expire_time; /* Expire time for the remote-session cookie */
+	zend_long     remote_cookie_expire_time; /* Expire time for the remote-session cookie */
 	char         *remote_addr_header; /* User configured header to check for forwarded IP address */
+	zend_long     remote_connect_timeout; /* Timeout in MS for remote connections */
 
 	char         *ide_key; /* As Xdebug uses it, from environment, USER, USERNAME or empty */
 	char         *ide_key_setting; /* Set through php.ini and friends */
@@ -269,7 +283,7 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	int           profile_last_functionname_ref;
 
 	/* DBGp globals */
-	char         *lastcmd;
+	const char   *lastcmd;
 	char         *lasttransid;
 
 	/* output redirection */
@@ -283,9 +297,24 @@ ZEND_BEGIN_MODULE_GLOBALS(xdebug)
 	zend_bool  do_scream;
 	zend_bool  in_at;
 
+	/* garbage stats */
+	zend_bool  gc_stats_enable;
+	zend_bool  gc_stats_enabled;
+	char      *gc_stats_output_dir;
+	char      *gc_stats_output_name;
+	FILE      *gc_stats_file;
+	char      *gc_stats_filename;
+
 	/* in-execution checking */
 	zend_bool  in_execution;
 	zend_bool  in_var_serialisation;
+
+	/* filters */
+	zend_long     filter_type_tracing;
+	zend_long     filter_type_profiler;
+	zend_long     filter_type_code_coverage;
+	xdebug_llist *filters_tracing;
+	xdebug_llist *filters_code_coverage;
 ZEND_END_MODULE_GLOBALS(xdebug)
 
 #ifdef ZTS
