@@ -2,18 +2,18 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2016 Derick Rethans                               |
+   | Copyright (c) 2002-2018 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 1.0 of the Xdebug license,    |
+   | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | http://xdebug.derickrethans.nl/license.php                           |
+   | https://xdebug.org/license.php                                       |
    | If you did not receive a copy of the Xdebug license and are unable   |
    | to obtain it through the world-wide-web, please send a note to       |
-   | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
+   | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors:  Derick Rethans <derick@xdebug.org>                         |
-   |           Thomas Vanhaniemi <thomas.vanhaniemi@arcada.fi>            |
+   | Authors: Derick Rethans <derick@xdebug.org>                          |
+   |          Thomas Vanhaniemi <thomas.vanhaniemi@arcada.fi>             |
    +----------------------------------------------------------------------+
  */
 #include "php_xdebug.h"
@@ -23,10 +23,18 @@
 #include <stdio.h>
 #include <fcntl.h>
 #ifndef PHP_WIN32
-# include <sys/poll.h>
+# if HAVE_POLL_H
+#  include <poll.h>
+# elif HAVE_SYS_POLL_H
+#  include <sys/poll.h>
+# endif
 # include <unistd.h>
 # include <sys/socket.h>
+# include <sys/un.h>
 # include <netinet/tcp.h>
+# if HAVE_NETINET_IN_H
+#  include <netinet/in.h>
+# endif
 # include <netdb.h>
 #else
 # include <process.h>
@@ -46,16 +54,43 @@
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
-int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
+#if !WIN32 && !WINNT
+static int xdebug_create_socket_unix(const char *path TSRMLS_DC)
+{
+	struct sockaddr_un sa;
+	int sockfd;
+
+	if ((sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) == SOCK_ERR) {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Creating socket for 'unix://%s', socket: %s.\n", path, strerror(errno));
+		return SOCK_ERR;
+	}
+
+	sa.sun_family = AF_UNIX;
+	strncpy(sa.sun_path, path, sizeof(sa.sun_path) - 1);
+	if (connect(sockfd, (struct sockaddr*)&sa, sizeof(sa)) < 0) {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Creating socket for 'unix://%s', connect: %s.\n", path, strerror(errno));
+		SCLOSE(sockfd);
+		return (errno == EACCES) ? SOCK_ACCESS_ERR : SOCK_ERR;
+	}
+
+	/* Prevent the socket from being inherited by exec'd children */
+	if (fcntl(sockfd, F_SETFD, FD_CLOEXEC) < 0) {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Creating socket for 'unix://%s', fcntl(FD_CLOEXEC): %s.\n", path, strerror(errno));
+	}
+
+	return sockfd;
+}
+#endif
+
+int xdebug_create_socket(const char *hostname, int dport, int timeout TSRMLS_DC)
 {
 	struct addrinfo            hints;
 	struct addrinfo            *remote;
 	struct addrinfo            *ptr;
 	int                        status;
-	int                        sockfd;
+	int                        sockfd = 0;
 	int                        sockerror;
 	char                       sport[10];
-	int                        timeout = 200;
 	int                        actually_connected;
 	struct sockaddr_in6        sa;
 	socklen_t                  size = sizeof(sa);
@@ -73,7 +108,16 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 	struct pollfd              ufds[1];
 	long                       optval = 1;
 #endif
-	
+
+	if (!strncmp(hostname, "unix://", strlen("unix://"))) {
+#if WIN32|WINNT
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Creating socket for '%s', Unix domain socket not supported.\n", hostname);
+		return SOCK_ERR;
+#else
+		return xdebug_create_socket_unix(hostname + strlen("unix://") TSRMLS_CC);
+#endif
+	}
+
 	/* Make a string of the port number that can be used with getaddrinfo */
 	sprintf(sport, "%d", dport);
 
@@ -117,10 +161,17 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 		fcntl(sockfd, F_SETFL, O_NONBLOCK);
 #endif
 
+#if !WIN32 && !WINNT
+		/* Prevent the socket from being inherited by exec'd children on *nix (not necessary on Win) */
+	        if (fcntl(sockfd, F_SETFD, FD_CLOEXEC) < 0) {
+        	        XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Creating socket for '%s:%d', fcntl(FD_CLOEXEC): %s.\n", hostname, dport, strerror(errno));
+        	}
+#endif
+
 		/* Try to connect to the newly created socket */
 		/* Worth noting is that the port is set in the getaddrinfo call before */
 		status = connect(sockfd, ptr->ai_addr, ptr->ai_addrlen);
-		
+
 		/* Determine if we got a connection. If no connection could be made
 		 * we close the socket and continue with the next IP address in the list */
 		if (status < 0) {
@@ -133,7 +184,7 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 				XDEBUG_LOG_PRINT(XG(remote_log_file), "W: Creating socket for '%s:%d', connect: %s.\n", hostname, dport, strerror(errno));
 				SCLOSE(sockfd);
 				sockfd = SOCK_ACCESS_ERR;
-				
+
 				continue;
 			}
 			if (errno != EINPROGRESS) {
@@ -141,7 +192,7 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 #endif
 				SCLOSE(sockfd);
 				sockfd = SOCK_ERR;
-				
+
 				continue;
 			}
 
@@ -171,13 +222,13 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 					sockerror = SOCK_ERR;
 					break;
 				}
-				
+
 				/* A timeout occured when polling the socket */
 				if (sockerror == 0) {
 					sockerror = SOCK_TIMEOUT_ERR;
 					break;
 				}
-				
+
 				/* If the poll was successful but an error occured */
 				if (ufds[0].revents & (POLLERR | POLLHUP | POLLNVAL)) {
 #if WIN32|WINNT
@@ -188,7 +239,7 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 					sockerror = SOCK_ERR;
 					break;
 				}
-				
+
 				/* If the poll was successful break out */
 				if (ufds[0].revents & (POLLIN | POLLOUT)) {
 					sockerror = sockfd;
@@ -216,16 +267,16 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 					sockerror = SOCK_ERR;
 				}
 			}
-			
+
 			/* If there where some errors close the socket and continue with the next IP address */
 			if (sockerror < 0) {
 				SCLOSE(sockfd);
 				sockfd = sockerror;
-				
+
 				continue;
 			}
 		}
-		
+
 		break;
 	}
 
@@ -245,11 +296,11 @@ int xdebug_create_socket(const char *hostname, int dport TSRMLS_DC)
 
 		setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, &optval, sizeof(optval));
 	}
-	
+
 	return sockfd;
 }
 
-void xdebug_close_socket(int socket)
+void xdebug_close_socket(int socketfd)
 {
-	SCLOSE(socket);
+	SCLOSE(socketfd);
 }
