@@ -2,17 +2,17 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2016 Derick Rethans                               |
+   | Copyright (c) 2002-2018 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 1.0 of the Xdebug license,    |
+   | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | http://xdebug.derickrethans.nl/license.php                           |
+   | https://xdebug.org/license.php                                       |
    | If you did not receive a copy of the Xdebug license and are unable   |
    | to obtain it through the world-wide-web, please send a note to       |
-   | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
+   | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors:  Derick Rethans <derick@xdebug.org>                         |
+   | Authors: Derick Rethans <derick@xdebug.org>                          |
    +----------------------------------------------------------------------+
  */
 
@@ -22,14 +22,14 @@
 /* Memory allocators */
 #if 0
 #define xdmalloc    emalloc
-#define xdcalloc    ecalloc	
+#define xdcalloc    ecalloc
 #define xdrealloc   erealloc
 #define xdfree      efree
 #define xdstrdup    estrdup
 #define xdstrndup   estrndup
-#else  
+#else
 #define xdmalloc    malloc
-#define xdcalloc    calloc	
+#define xdcalloc    calloc
 #define xdrealloc   realloc
 #define xdfree      free
 #define xdstrdup    strdup
