@@ -2,15 +2,15 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2016 Derick Rethans                               |
+   | Copyright (c) 2002-2018 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 1.0 of the Xdebug license,    |
+   | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | http://xdebug.derickrethans.nl/license.php                           |
+   | https://xdebug.org/license.php                                       |
    | If you did not receive a copy of the Xdebug license and are unable   |
    | to obtain it through the world-wide-web, please send a note to       |
-   | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
+   | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
    | Authors: Derick Rethans <derick@xdebug.org>                          |
    +----------------------------------------------------------------------+
@@ -21,13 +21,13 @@
 
 extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 
-void *xdebug_trace_computerized_init(char *fname, long options TSRMLS_DC)
+void *xdebug_trace_computerized_init(char *fname, char *script_filename, long options TSRMLS_DC)
 {
 	xdebug_trace_computerized_context *tmp_computerized_context;
 	char *used_fname;
 
 	tmp_computerized_context = xdmalloc(sizeof(xdebug_trace_computerized_context));
-	tmp_computerized_context->trace_file = xdebug_trace_open_file(fname, options, (char**) &used_fname TSRMLS_CC);
+	tmp_computerized_context->trace_file = xdebug_trace_open_file(fname, script_filename, options, (char**) &used_fname TSRMLS_CC);
 	tmp_computerized_context->trace_filename = used_fname;
 
 	return tmp_computerized_context->trace_file ? tmp_computerized_context : NULL;
@@ -89,26 +89,30 @@ char *xdebug_trace_computerized_get_filename(void *ctxt TSRMLS_DC)
 	return context->trace_filename;
 }
 
-static char *render_variable(zval *var, int type TSRMLS_DC)
+static void add_single_value(xdebug_str *str, zval *zv, int collection_level TSRMLS_DC)
 {
-	char *tmp_value = NULL;
+	xdebug_str *tmp_value = NULL;
 
-	switch (XG(collect_params)) {
+	switch (collection_level) {
 		case 1: /* synopsis */
 		case 2:
-			tmp_value = xdebug_get_zval_synopsis(var, 0, NULL);
+			tmp_value = xdebug_get_zval_synopsis(zv, 0, NULL);
 			break;
 		case 3: /* full */
 		case 4: /* full (with var) */
 		default:
-			tmp_value = xdebug_get_zval_value(var, 0, NULL);
+			tmp_value = xdebug_get_zval_value(zv, 0, NULL);
 			break;
 		case 5: /* serialized */
-			tmp_value = xdebug_get_zval_value_serialized(var, 0, NULL TSRMLS_CC);
+			tmp_value = xdebug_get_zval_value_serialized(zv, 0, NULL);
 			break;
 	}
-
-	return tmp_value;
+	if (tmp_value) {
+		xdebug_str_add_str(str, tmp_value);
+		xdebug_str_free(tmp_value);
+	} else {
+		xdebug_str_add(str, "???", 0);
+	}
 }
 
 
@@ -132,21 +136,12 @@ void xdebug_trace_computerized_function_entry(void *ctxt, function_stack_entry *
 
 	if (fse->include_filename) {
 		if (fse->function.type == XFUNC_EVAL) {
-#if PHP_VERSION_ID >= 70000
 			zend_string *i_filename = zend_string_init(fse->include_filename, strlen(fse->include_filename), 0);
 			zend_string *escaped;
-			escaped = php_addcslashes(i_filename, 0, "'\\\0..\37", 6);
+			escaped = php_addcslashes(i_filename, 0, (char*) "'\\\0..\37", 6);
 			xdebug_str_add(&str, xdebug_sprintf("'%s'", escaped->val), 1);
 			zend_string_release(escaped);
 			zend_string_release(i_filename);
-#else
-			int tmp_len;
-
-			char *escaped;
-			escaped = php_addcslashes(fse->include_filename, strlen(fse->include_filename), &tmp_len, 0, "'\\\0..\37", 6 TSRMLS_CC);
-			xdebug_str_add(&str, xdebug_sprintf("'%s'", escaped), 1);
-			efree(escaped);
-#endif
 		} else {
 			xdebug_str_add(&str, fse->include_filename, 0);
 		}
@@ -164,8 +159,6 @@ void xdebug_trace_computerized_function_entry(void *ctxt, function_stack_entry *
 
 		/* Arguments (12-...) */
 		for (j = 0; j < fse->varc; j++) {
-			char *tmp_value;
-
 			xdebug_str_addl(&str, "\t", 1, 0);
 
 			if (fse->var[j].is_variadic) {
@@ -176,10 +169,8 @@ void xdebug_trace_computerized_function_entry(void *ctxt, function_stack_entry *
 				xdebug_str_add(&str, xdebug_sprintf("$%s = ", fse->var[j].name), 1);
 			}
 
-			tmp_value = render_variable(fse->var[j].addr, XG(collect_params) TSRMLS_CC);
-
-			if (tmp_value) {
-				xdebug_str_add(&str, tmp_value, 1);
+			if (!Z_ISUNDEF(fse->var[j].data)) {
+				add_single_value(&str, &(fse->var[j].data), XG(collect_params));
 			} else {
 				xdebug_str_add(&str, "???", 0);
 			}
@@ -188,7 +179,7 @@ void xdebug_trace_computerized_function_entry(void *ctxt, function_stack_entry *
 
 	/* Trailing \n */
 	xdebug_str_add(&str, "\n", 0);
-	
+
 	fprintf(context->trace_file, "%s", str.d);
 	fflush(context->trace_file);
 	xdfree(str.d);
@@ -215,19 +206,13 @@ void xdebug_trace_computerized_function_return_value(void *ctxt, function_stack_
 {
 	xdebug_trace_computerized_context *context = (xdebug_trace_computerized_context*) ctxt;
 	xdebug_str str = XDEBUG_STR_INITIALIZER;
-	char      *tmp_value = NULL;
 
 	xdebug_str_add(&str, xdebug_sprintf("%d\t", fse->level), 1);
 	xdebug_str_add(&str, xdebug_sprintf("%d\t", function_nr), 1);
 	xdebug_str_add(&str, "R\t\t\t", 0);
 
-	tmp_value = render_variable(return_value, XG(collect_params) TSRMLS_CC);
+	add_single_value(&str, return_value, XG(collect_params));
 
-	if (tmp_value) {
-		xdebug_str_add(&str, tmp_value, 1);
-	} else {
-		xdebug_str_add(&str, "???", 0);
-	}
 	xdebug_str_addl(&str, "\n", 2, 0);
 
 	fprintf(context->trace_file, "%s", str.d);
@@ -245,8 +230,6 @@ xdebug_trace_handler_t xdebug_trace_handler_computerized =
 	xdebug_trace_computerized_function_entry,
 	xdebug_trace_computerized_function_exit,
 	xdebug_trace_computerized_function_return_value,
-#if PHP_VERSION_ID >= 50500
 	NULL /* xdebug_trace_computerized_generator_return_value */,
-#endif
 	NULL /* xdebug_trace_computerized_assignment */
 };
